@@ -1,5 +1,5 @@
-phantom_run <- function(args, wait = TRUE) {
-  phantom_bin <- find_phantom()
+phantom_run <- function(args, wait = TRUE, quiet = FALSE) {
+  phantom_bin <- find_phantom(quiet = quiet)
 
   # Handle missing phantomjs
   if (is.null(phantom_bin)) return(NULL)
@@ -7,12 +7,34 @@ phantom_run <- function(args, wait = TRUE) {
   # Make sure args is a char vector
   args <- as.character(args)
 
-  system2(phantom_bin, args = args, wait = wait)
+  p <- callr::process$new(
+    phantom_bin,
+    args = args,
+    stdout = "|", stderr = "|",
+    supervise = TRUE
+  )
+
+  if (isTRUE(wait)) {
+    on.exit({
+      p$kill()
+    })
+    cat_n <- function(txt) {
+      if (length(txt) > 0) {
+        cat(txt, sep = "\n")
+      }
+    }
+    while(p$is_alive()) {
+      p$wait(200) # wait until min(c(time_ms, process ends))
+      cat_n(p$read_error_lines())
+      cat_n(p$read_output_lines())
+    }
+  }
+  p$get_exit_status()
 }
 
 
 # Find PhantomJS from PATH, APPDATA, system.file('webshot'), ~/bin, etc
-find_phantom <- function() {
+find_phantom <- function(quiet = FALSE) {
   path <- Sys.which( "phantomjs" )
   if (path != "") return(path)
 
@@ -28,43 +50,114 @@ find_phantom <- function() {
     # and may not be capable of installing phantomjs (like on Solaris), and any
     # packages which use webshot in their R CMD check (in examples or vignettes)
     # will get an ERROR. We'll issue a message and return NULL; other
-    message(
-      "PhantomJS not found. You can install it with webshot::install_phantomjs(). ",
-      "If it is installed, please make sure the phantomjs executable ",
-      "can be found via the PATH variable."
-    )
+
+    if(!quiet) {
+      message(
+        "PhantomJS not found. You can install it with webshot::install_phantomjs(). ",
+        "If it is installed, please make sure the phantomjs executable ",
+        "can be found via the PATH variable."
+      )
+    }
     return(NULL)
   }
   path.expand(path)
 }
 
+phantomjs_cmd_result <- function(args, wait = TRUE, quiet = FALSE) {
+  # Retrieve and store output from STDOUT
+  utils::capture.output(invisible(phantom_run(args = args, wait = wait, quiet = quiet)),
+                        type = "output")
+}
+
+phantomjs_version <- function() {
+  phantomjs_cmd_result("--version", quiet = TRUE)
+}
+
+#' Determine if PhantomJS is Installed
+#'
+#' Verifies that a version of PhantomJS is installed and available for use
+#' on the user's computer.
+#'
+#' @return \code{TRUE} if the PhantomJS is installed. Otherwise, \code{FALSE}
+#' if PhantomJS is not installed.
+#'
+#' @export
+is_phantomjs_installed <- function() {
+  !is.null(find_phantom(quiet = TRUE))
+}
+
+is_phantomjs_version_latest <- function(requested_version) {
+  # Ensure phantomjs is installed if not, request an update
+  if (!is_phantomjs_installed()) {
+    return(FALSE)
+  }
+
+  # Obtain the installed version
+  installed_phantomjs_version <- phantomjs_version()
+
+  # For versions 2.5.0-beta and 2.5.0-beta2, phantomjs reports its version to be
+  # "2.5.0-development".
+  #
+  # However, for the requested version, we have the names "2.5.0-beta" and
+  # "2.5.0-beta2".
+  #
+  # For simplicity, we'll just remove the "-development" or "-beta" or "-beta2"
+  # so they all map to "2.5.0".
+  installed_phantomjs_version <- sub("-development", "", installed_phantomjs_version)
+  requested_version           <- sub("-beta.*",      "", requested_version)
+
+  # Check if the installed version is latest compared to requested version.
+  as.package_version(installed_phantomjs_version) >= requested_version
+}
+
+
 #' Install PhantomJS
 #'
 #' Download the zip package, unzip it, and copy the executable to a system
 #' directory in which \pkg{webshot} can look for the PhantomJS executable.
 #'
-#' This function was designed primarily to help Windows users since it is
-#' cumbersome to modify the \code{PATH} variable. Mac OS X users may install
-#' PhantomJS via Homebrew. If you download the package from the PhantomJS
-#' website instead, please make sure the executable can be found via the
-#' \code{PATH} variable.
+#' @details This function was designed primarily to help Windows users since it
+#'   is cumbersome to modify the \code{PATH} variable. Mac OS X users may
+#'   install PhantomJS via Homebrew. If you download the package from the
+#'   PhantomJS website instead, please make sure the executable can be found via
+#'   the \code{PATH} variable.
+#'
+#'   On Windows, the directory specified by the environment variable
+#'   \code{APPDATA} is used to store \file{phantomjs.exe}. On OS X, the
+#'   directory \file{~/Library/Application Support} is used. On other platforms
+#'   (such as Linux), the directory \file{~/bin} is used. If these directories
+#'   are not writable, the directory \file{PhantomJS} under the installation
+#'   directory of the \pkg{webshot} package will be tried. If this directory
+#'   still fails, you will have to install PhantomJS by yourself.
+#'
+#'   If PhantomJS is not already installed on the computer, this function will
+#'   attempt to install it. However, if the version of PhantomJS installed is
+#'   greater than or equal to the requested version, this function will not
+#'   perform the installation procedure again unless the \code{force} parameter
+#'   is set to \code{TRUE}. As a result, this function may also be used to
+#'   reinstall or downgrade the version of PhantomJS found.
 #'
-#' On Windows, the directory specified by the environment variable
-#' \code{APPDATA} is used to store \file{phantomjs.exe}. On OS X, the directory
-#' \file{~/Library/Application Support} is used. On other platforms (such as
-#' Linux), the directory \file{~/bin} is used. If these directories are not
-#' writable, the directory \file{PhantomJS} under the installation directory of
-#' the \pkg{webshot} package will be tried. If this directory still fails, you
-#' will have to install PhantomJS by yourself.
 #' @param version The version number of PhantomJS.
 #' @param baseURL The base URL for the location of PhantomJS binaries for
 #'   download. If the default download site is unavailable, you may specify an
 #'   alternative mirror, such as
 #'   \code{"https://bitbucket.org/ariya/phantomjs/downloads/"}.
+#' @param force Install PhantomJS even if the version installed is the latest or
+#'   if the requested version is older. This is useful to reinstall or downgrade
+#'   the version of PhantomJS.
 #' @return \code{NULL} (the executable is written to a system directory).
 #' @export
 install_phantomjs <- function(version = '2.1.1',
-    baseURL = 'https://github.com/wch/webshot/releases/download/v0.3.1/') {
+    baseURL = 'https://github.com/wch/webshot/releases/download/v0.3.1/',
+    force = FALSE) {
+
+  if (!force && is_phantomjs_version_latest(version)) {
+      message('It seems that the version of `phantomjs` installed is ',
+              'greater than or equal to the requested version.',
+              'To install the requested version or downgrade to another version, ',
+              'use `force = TRUE`.')
+      return(invisible())
+  }
 
   if (!grepl("/$", baseURL))
     baseURL <- paste0(baseURL, "/")
@@ -126,7 +219,7 @@ phantom_paths <- function() {
   } else {
     path <- '~/bin'
   }
-  path <- c(path, system.file('PhantomJS', package = 'webshot'))
+  path <- c(path, file.path(system.file(package = 'webshot'), 'PhantomJS'))
   path
 }
 
@@ -259,8 +352,10 @@ fix_windows_url <- function(url) {
   fix_one <- function(x) {
     # If it's a "c:/path/file.html" path, or contains any backslashs, like
     # "c:\path", "\\path\\file.html", or "/path\\file.html", we need to fix it
-    # up.
-    if (grepl("^[a-zA-Z]:/", x) || grepl("\\", x, fixed = TRUE)) {
+    # up. However, we need to leave paths that are already URLs alone.
+    if (grepl("^[a-zA-Z]:/", x) ||
+        (!grepl(":", x, fixed = TRUE) && grepl("\\", x, fixed = TRUE)))
+    {
       paste0("file:///", normalizePath(x, winslash = "/"))
     } else {
       x
